# Copyright (c) 2019 Workonline Communications (Pty) Ltd. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""prefix_list_agent agent implementation."""

from __future__ import print_function

import collections
import datetime
import filecmp
import os
import shutil
import signal
import tempfile

import eossdk

from prefix_list_agent.base import PrefixListBase
from prefix_list_agent.worker import PrefixListWorker


class PrefixListAgent(PrefixListBase, eossdk.AgentHandler,
                      eossdk.TimeoutHandler, eossdk.FdHandler):
    """An EOS SDK based agent that creates prefix-list policy objects."""

    sysdb_mounts = ("agent",)
    agent_options = ("rptk_endpoint", "source_dir", "refresh_interval")

    @classmethod
    def set_sysdb_mp(cls, name):
        """Create the SysdbMountProfiles file for the agent."""
        # set the path
        profile_path = os.path.join("/usr/lib/SysdbMountProfiles", name)
        # get a tempfile for writing the profile to
        with tempfile.NamedTemporaryFile() as tmp:
            # write the profile file
            tmp.write("agentName:{}-%sliceId\n\n".format(name))
            for profile in cls.sysdb_mounts:
                tmp.write("Include: EosSdk_{}.include\n".format(profile))
            tmp.flush()
            # check whether an existing file matches and bail out
            if os.path.isfile(profile_path):
                if filecmp.cmp(profile_path, tmp.name, shallow=False):
                    return False
            # copy the tempfile into place
            shutil.copy(tmp.name, profile_path)
        return True

    def __init__(self, sdk):
        """Initialise the agent instance."""
        # Set up tracing
        PrefixListBase.__init__(self)
        # get sdk managers
        self.agent_mgr = sdk.get_agent_mgr()
        self.timeout_mgr = sdk.get_timeout_mgr()
        self.eapi_mgr = sdk.get_eapi_mgr()
        # init sdk handlers
        eossdk.AgentHandler.__init__(self, self.agent_mgr)
        eossdk.TimeoutHandler.__init__(self, self.timeout_mgr)
        eossdk.FdHandler.__init__(self)
        # set worker process to None
        self.worker = None
        self.watching = set()
        # set default confg options
        self._rptk_endpoint = None
        self._source_dir = "/tmp/prefix-lists"
        self._refresh_interval = 3600
        # create state containers
        self._status = None
        self._last_start = None
        self._last_end = None
        self._result = None

    @property
    def rptk_endpoint(self):
        """Get 'rptk_endpoint' property."""
        return self._rptk_endpoint

    @rptk_endpoint.setter
    def rptk_endpoint(self, url):
        """Set 'rptk_endpoint' property."""
        self._rptk_endpoint = url

    @property
    def source_dir(self):
        """Get 'source_dir' property."""
        return self._source_dir

    @source_dir.setter
    def source_dir(self, path):
        """Set 'source_dir' property."""
        self._source_dir = path

    @property
    def refresh_interval(self):
        """Get 'refresh_interval' property."""
        return self._refresh_interval

    @refresh_interval.setter
    def refresh_interval(self, i):
        """Set 'refresh_interval' property."""
        i = int(i)
        if i in range(10, 86400):
            self._refresh_interval = i
        else:
            raise ValueError("refresh_interval must be in range 1 - 86399")

    @property
    def status(self):
        """Get 'status' property."""
        return self._status

    @status.setter
    def status(self, s):
        """Set 'status' property."""
        self._status = s
        self.agent_mgr.status_set("status", self.status)
        self.info("Status: {}".format(self.status))

    @property
    def result(self):
        """Get 'result' property."""
        return self._result

    @result.setter
    def result(self, r):
        """Set 'result' property."""
        self._result = r
        self.agent_mgr.status_set("result", self.result)
        self.notice("Result: {}".format(self.result))

    @property
    def last_start(self):
        """Set the 'last_start' timestamp."""
        return self._last_start

    @last_start.setter
    def last_start(self, ts):
        """Set the 'last_start' timestamp."""
        if not isinstance(ts, datetime.datetime):
            raise TypeError("Expected datetime.datetime, got {}".format(ts))
        self._last_start = ts
        self.agent_mgr.status_set("last_start", str(self.last_start))
        self.info("Last start: {}".format(ts))

    @property
    def last_end(self):
        """Set the 'last_end' timestamp."""
        return self._last_end

    @last_end.setter
    def last_end(self, ts):
        """Set the 'last_end' timestamp."""
        if not isinstance(ts, datetime.datetime):
            raise TypeError("Expected datetime.datetime, got {}".format(ts))
        self._last_end = ts
        self.agent_mgr.status_set("last_end", str(self.last_end))
        self.info("Last end: {}".format(ts))

    def configure(self):
        """Read and set all configuration options."""
        self.info("Reading configuration options")
        for key in self.agent_mgr.agent_option_iter():
            value = self.agent_mgr.agent_option(key)
            self.set(key, value)

    def set(self, key, value):
        """Set a configuration option."""
        if not value:
            value = None
        self.info("Setting configuration '{}'='{}'".format(key, value))
        if key in self.agent_options:
            setattr(self, key, value)
        else:
            self.warning("Ignoring unknown option '{}'".format(key))

    def start(self):
        """Start up the agent."""
        self.status = "init"
        self.configure()
        self.init()
        self.run()

    def init(self):  # pragma: no cover
        """Perform one-time start actions."""
        pass

    def run(self):
        """Spawn worker process."""
        self.status = "running"
        if self.rptk_endpoint is not None:
            self.last_start = datetime.datetime.now()
            try:
                self.info("Initialising worker")
                self.worker = PrefixListWorker(endpoint=self.rptk_endpoint,
                                               path=self.source_dir,
                                               eapi=self.eapi_mgr)
                self.watch(self.worker.p_data, "result")
                self.watch(self.worker.p_err, "error")
                self.info("Starting worker")
                self.worker.start()
                self.info("Worker started: pid {}".format(self.worker.pid))
            except Exception as e:
                self.err("Starting worker failed: {}".format(e))
                self.failure(err=e)
        else:
            self.warning("'rptk_endpoint' is not set"
                         .format(self.rptk_endpoint))
            self.sleep()

    def watch(self, conn, type):
        """Watch a Connection for new data."""
        self.info("Trying to watch for {} data on {}".format(type, conn))
        fileno = conn.fileno()
        self.watch_readable(fileno, True)
        self.watching.add(conn)
        self.info("Watching {} for {} data".format(conn, type))

    def unwatch(self, conn, close=False):
        """Stop watching a Connection for new data."""
        self.info("Trying to remove watch on {}".format(conn))
        fileno = conn.fileno()
        self.watch_readable(fileno, False)
        if conn in self.watching:
            self.watching.remove(conn)
        self.info("Stopped watching {}".format(conn))
        if close:
            self.info("Closing connection {}".format(conn))
            conn.close()

    def success(self):
        """Report statistics and restart refresh_interval timer."""
        self.status = "finalising"
        self.info("Receiving results from worker")
        stats = self.worker.data
        self.report(**stats)
        self.result = "ok"
        self.last_end = datetime.datetime.now()
        self.cleanup(process=self.worker)
        self.sleep()

    def failure(self, err=None, process=None, restart=False):
        """Handle worker exception."""
        self.status = "error"
        if err is None:
            try:
                err = process.error
            except Exception as e:
                self.err("Retreiving exception from {} failed"
                         .format(process.__class__.__name__))
                err = e
        self.err(err)
        self.result = "failed"
        self.last_end = datetime.datetime.now()
        if restart:
            self.restart()
        else:
            self.cleanup(process=process)
            self.sleep()

    def report(self, **stats):
        """Report statistics to the agent manager."""
        for name, value in stats.items():
            self.info("{}: {}".format(name, value))
            self.agent_mgr.status_set(name, str(value))

    def cleanup(self, process):
        """Kill the process if it is still running."""
        self.status = "cleanup"
        process_name = process.__class__.__name__
        self.info("Cleaning up {} process".format(process_name))
        if process is not None:
            self.info("Closing connections from {}".format(process_name))
            for conn in [c for c in
                         [getattr(process, k) for k in dir(process)
                          if not k.startswith("_")]
                         if isinstance(c, collections.Hashable)
                         and c in self.watching]:
                try:
                    self.unwatch(conn, close=True)
                except Exception as e:
                    self.err(e)
            for retry in range(3):
                if process.is_alive():
                    if retry:
                        self.info("Sending {} SIGTERM".format(process_name))
                        process.terminate()
                    timeout = retry * 5 + 1
                    self.info("Waiting {} seconds for {} (pid: {}) to join"
                              .format(timeout, process_name, process.pid))
                    process.join(timeout)
            if process.is_alive():
                self.notice("Timeout waiting for {}. Sending SIGKILL"
                            .format(process_name))
                os.kill(process.pid, signal.SIGKILL)
        self.info("Cleanup complete")

    def sleep(self):
        """Go to sleep for 'refresh_interval' seconds."""
        self.status = "sleeping"
        self.timeout_time_is(eossdk.now() + self.refresh_interval)

    def shutdown(self):
        """Shutdown the agent gracefully."""
        self.notice("Shutting down")
        try:
            self.cleanup(process=self.worker)
        except Exception as e:
            self.err(e)
        self.status = "shutdown"
        self.agent_mgr.agent_shutdown_complete_is(True)

    def restart(self):
        """Restart the agent."""
        self.notice("Restarting")
        self.status = "restarting"
        try:
            self.cleanup(process=self.worker)
        except Exception as e:
            self.err(e)
        self.start()

    def on_initialized(self):
        """Start the agent after initialisation."""
        self.start()

    def on_agent_option(self, key, value):
        """Handle a change to a configuration option."""
        self.set(key, value)

    def on_agent_enabled(self, enabled):
        """Handle a change in the admin state of the agent."""
        if enabled:
            self.notice("Agent enabled")
        else:
            self.notice("Agent disabled")
            self.shutdown()

    def on_timeout(self):
        """Handle a 'refresh_interval' timeout."""
        self.run()

    def on_readable(self, fd):
        """Handle a watched file descriptor becoming readable."""
        self.info("Watched file descriptor {} is readable".format(fd))
        if fd == self.worker.p_data.fileno():
            self.info("Data channel is ready")
            return self.success()
        elif fd == self.worker.p_err.fileno():
            self.info("Exception received from worker")
            return self.failure(process=self.worker)
        else:
            self.warning("Unknown file descriptor: ignoring")



#!/usr/bin/env python
# Copyright (c) 2019 Workonline Communications (Pty) Ltd. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""prefix_list_agent package metadata."""

from __future__ import print_function
from __future__ import unicode_literals

__version__ = "0.1.0rc3"
__author__ = "Ben Maddison"
__author_email__ = "benm@workonline.africa"
__licence__ = "MIT"
__copyright__ = "Copyright (c) 2019 Workonline Communications (Pty) Ltd"
__url__ = "https://github.com/wolcomm/eos-prefix-list-agent"
__classifiers__ = [
    "Development Status :: 4 - Beta",
    "Environment :: Other Environment",
    "Intended Audience :: Telecommunications Industry",
    "License :: OSI Approved :: MIT License",
    "Operating System :: POSIX :: Other",
    "Programming Language :: Python :: 2.7",
    "Topic :: Internet",
    "Topic :: System :: Networking"
]
__entry_points__ = None
__scripts__ = [
    "bin/PrefixListAgent"
]


if __name__ == "__main__":
    print(__version__)



# Copyright (c) 2019 Ben Maddison. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""prefix_list_agent worker functions."""

from __future__ import print_function

import collections
import json
import multiprocessing
import os
import re
import signal
import sys
import urllib2

from prefix_list_agent.base import PrefixListBase
from prefix_list_agent.exceptions import handle_sigterm, TermException


class PrefixListWorker(multiprocessing.Process, PrefixListBase):
    """Worker to fetch and process IRR data."""

    def __init__(self, endpoint, path, eapi, *args, **kwargs):
        """Initialise an PrefixListWorker instance."""
        super(PrefixListWorker, self).__init__(*args, **kwargs)
        PrefixListBase.__init__(self)
        self.endpoint = endpoint
        self.path = path
        self.eapi = eapi
        self.path_re = re.compile(r"^file:{}/(?P<policy>\w+)/(?P<file>[-.:\w]+)$"  # noqa: E501
                                  .format(self.path.rstrip("/")))
        self._p_err, self._c_err = multiprocessing.Pipe(duplex=False)
        self._p_data, self._c_data = multiprocessing.Pipe(duplex=False)

    @property
    def p_err(self):
        """Get 'p_err' property."""
        return self._p_err

    @property
    def c_err(self):
        """Get 'c_err' property."""
        return self._c_err

    @property
    def p_data(self):
        """Get 'p_data' property."""
        return self._p_data

    @property
    def c_data(self):
        """Get 'c_data' property."""
        return self._c_data

    def run(self):
        """Run the worker process."""
        self.info("Worker started")
        signal.signal(signal.SIGTERM, handle_sigterm)
        try:
            policies = self.get_policies()
            configured = self.get_configured(policies)
            data = self.get_data(configured)
            stats = self.write_results(configured, data)
            self.refresh_all()
            self.c_data.send(stats)
        except TermException:
            self.notice("Got SIGTERM signal: exiting.")
            if os.getpid() == self.pid:
                sys.exit(127 + signal.SIGTERM)
        except Exception as e:
            self.err(e)
            try:
                self.c_err.send(e)
            except TypeError:  # pragma: no cover
                self.c_err.send(Exception(str(e)))
        finally:
            self.c_err.close()
            self.c_data.close()

    def get_configured(self, policies):
        """Get the prefix-lists in running-config."""
        configured = {p: collections.defaultdict(dict) for p in policies}
        self.info("Searching for prefix-lists with source matching {}"
                  .format(self.path_re.pattern))
        for afi, cmd in (("ipv4", "show ip prefix-list"),
                         ("ipv6", "show ipv6 prefix-list")):
            data = self.eapi_request(cmd, result_node="ipPrefixLists")
            self.debug("Got response: {}".format(data))
            for name, config in data.items():
                try:
                    source = config["ipPrefixListSource"]
                except KeyError:
                    continue
                self.debug("Testing {}, source {}".format(name, source))
                m = self.path_re.match(source)
                if m:
                    self.debug("Source matched")
                    (policy, file) = m.groups()
                    if policy in policies:
                        configured[policy][name][afi] = file
                    else:
                        self.warning("Ignoring unknown policy {}"
                                     .format(policy))
                else:
                    self.debug("No match")
        return configured

    def refresh_all(self):
        """Refresh prefix-lists."""
        self.info("Refreshing source-based prefix-lists")
        for afi in ("ip", "ipv6"):
            cmd = "refresh {} prefix-list".format(afi)
            messages = self.eapi_request(cmd, result_node="messages",
                                         allow_empty=True)
            for msg in messages:
                for submsg in msg.replace("\nNum", " -").rstrip().split("\n"):
                    self.info(submsg)
        self.notice("Prefix-lists refreshed successfully")

    def get_policies(self):
        """Get the list of valid policy names from RPTK."""
        url_path = "/policies"
        self.info("Trying to get policy data from {}".format(url_path))
        policies = self.rptk_request(url_path)
        self.debug("Got policies: {}".format(policies.keys()))
        return policies

    def get_data(self, configured):
        """Get IRR data for the configured prefix-list objects."""
        data = dict()
        self.info("Querying for IRR data")
        for policy, objs in configured.items():
            if not objs:
                continue
            self.info("Trying bulk query")
            try:
                result = self.get_data_bulk(policy, objs)
                data.update({policy: result})
                continue
            except Exception as e:
                self.err(e)
            self.info("Failing back to indiviual queries")
            data[policy] = dict()
            for obj in objs:
                try:
                    result = self.get_data_obj(policy, obj)
                except Exception as e:
                    self.err(e)
                    continue
                data[policy].update(result)
        return data

    def get_data_bulk(self, policy, objs):
        """Get IRR data in bulk."""
        url_path = "/json/query?policy={}&".format(policy) + \
                   "&".join(["objects={}".format(obj) for obj in objs])
        self.info("Trying to get prefix data from {}".format(url_path))
        result = self.rptk_request(url_path)
        self.debug("Got prefix data")
        return result

    def get_data_obj(self, policy, obj):
        """Get IRR data for a single object."""
        url_path = "/json/{}/{}".format(obj, policy)
        self.info("Trying to get prefix data from {}".format(url_path))
        result = self.rptk_request(url_path)
        self.debug("Got prefix data")
        return result

    def write_results(self, configured, data):
        """Write prefix-list data to files."""
        stats = {"succeeded": 0, "failed": 0}
        for policy, objs in configured.items():
            self.info("Writing files for policy {}".format(policy))
            if not objs:
                self.info("No objects with policy: {}".format(policy))
                continue
            policy_dir = os.path.join(self.path, policy)
            if not os.path.isdir(policy_dir):
                self.info("Creating directory {}".format(policy_dir))
                os.makedirs(policy_dir)
            for obj, config in objs.items():
                self.info("Trying to write files for {}/{}"
                          .format(obj, policy))
                if obj in data[policy]:
                    for afi, file in config.items():
                        path = os.path.join(policy_dir, file)
                        entries = data[policy][obj][afi]
                        try:
                            self.write_prefix_list(path, entries, afi)
                        except Exception:  # pragma: no cover
                            stats["failed"] += 1
                            continue
                        stats["succeeded"] += 1
                else:
                    self.warning("No prefix data for {}/{}"
                                 .format(obj, policy))
                    stats["failed"] += len(config)
        return stats

    def write_prefix_list(self, path, entries, afi):
        """Write prefix-list to file."""
        self.info("Trying to write {}".format(path))
        try:
            with open(path, "w") as f:
                for i, p in enumerate(entries):
                    f.write(self.prefix_list_line(i, p))
        except Exception as e:
            self.err("Failed to write {}: {}".format(path, e))
            raise e

    def prefix_list_line(self, index, entry):
        """Generate a line in a prefix-list."""
        line = "seq {} permit {}".format(index + 1, entry["prefix"])
        if not entry["exact"]:
            if "greater-equal" in entry:
                line += " ge {}".format(entry["greater-equal"])
            if "less-equal" in entry:
                line += " le {}".format(entry["less-equal"])
        line += "\n"
        return line

    def eapi_request(self, cmd, result_node, allow_empty=False):
        """Get call an enable-mode eAPI command."""
        self.debug("Calling eAPI command {}".format(cmd))
        try:
            resp = self.eapi.run_show_cmd(cmd)
        except Exception as e:
            self.err("eAPI request failed: {}".format(e))
            raise e
        if resp.success():
            data = self.json_load(resp.responses()[0])
        else:
            e = RuntimeError("eAPI request failed: {} ({})"
                             .format(resp.error_message(),
                                     resp.error_code()))
            self.err(e)
            raise e
        try:
            result = data[result_node]
        except KeyError as e:
            if allow_empty:
                result = {}
            else:
                self.err("Failed to get result data: {}".format(e))
                raise e
        self.debug("eAPI request successful")
        return result

    def rptk_request(self, url_path):
        """Perform a query against the RPTK endpoint."""
        url = "{}/{}".format(self.endpoint.rstrip("/"),
                             url_path.lstrip("/"))
        self.debug("Querying RPTK endpoint at {}".format(url))
        try:
            resp = urllib2.urlopen(url)
        except urllib2.HTTPError as e:
            self.err("Request failed: {} {}".format(e.code, e.reason))
            raise e
        except urllib2.URLError as e:
            self.err("Request failed: {}".format(e))
            raise e
        self.debug("Request successful: {}".format(resp.getcode()))
        result = self.json_load(resp)
        return result

    def json_load(self, obj):
        """Deserialise JSON from a string or file-like object."""
        def fail(e):
            self.err("Failed to deserialise response: {}".format(e))
            raise e

        self.debug("Deserialising JSON response")
        try:
            self.debug("Trying 'json.load' method")
            result = json.load(obj)
        except AttributeError:
            self.debug("Object has no 'read' method")
            self.debug("Trying 'json.loads' method")
            try:
                result = json.loads(obj)
            except Exception as e:
                fail(e)
        except Exception as e:
            fail(e)
        self.debug("Successfully deserialised JSON")
        return result

    @property
    def data(self):
        """Get data from the worker."""
        if self.p_data.poll():
            return self.p_data.recv()

    @property
    def error(self):
        """Get exception raised by worker."""
        if self.p_err.poll():
            return self.p_err.recv()



# Copyright (c) 2019 Workonline Communications (Pty) Ltd. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""prefix_list_agent cli entry point."""

from __future__ import print_function

import sys

from prefix_list_agent import PrefixListAgent


def start(sdk):
    """Start the agent."""
    try:
        # create a Sysdb mount profile, restarting if necessary
        if PrefixListAgent.set_sysdb_mp(sdk.name()):
            # return a user defined status to indicate
            # that a restart is desired
            return 64
        # create an instance of the agent
        agent = PrefixListAgent(sdk)  # noqa: W0612
        # enter the sdk event-loop
        sdk.main_loop(sys.argv)
    except KeyboardInterrupt:
        return 130
    except Exception:
        raise
    return



# Copyright (c) 2019 Workonline Communications (Pty) Ltd. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""prefix_list_agent base class."""

from __future__ import print_function

import eossdk


class PrefixListBase(object):
    """Base class that implements tracing."""

    def __init__(self):
        """Initialise a PrefixListBase instance."""
        self.tracer = eossdk.Tracer(self.__class__.__name__)

    def _trace(self, msg, level=0):
        """Write tracing output."""
        self.tracer.trace(level, str(msg))

    def emerg(self, msg):
        """Write trace output at 'emergency' (0) level."""
        self._trace(msg, level=0)

    def alert(self, msg):
        """Write trace output at 'alert' (1) level."""
        self._trace(msg, level=1)

    def crit(self, msg):
        """Write trace output at 'critical' (2) level."""
        self._trace(msg, level=2)

    def err(self, msg):
        """Write trace output at 'error' (3) level."""
        self._trace(msg, level=3)

    def warning(self, msg):
        """Write trace output at 'warning' (4) level."""
        self._trace(msg, level=4)

    def notice(self, msg):
        """Write trace output at 'notice' (5) level."""
        self._trace(msg, level=5)

    def info(self, msg):
        """Write trace output at 'informational' (6) level."""
        self._trace(msg, level=6)

    def debug(self, msg):
        """Write trace output at 'debug' (7) level."""
        self._trace(msg, level=7)



# Copyright (c) 2019 Workonline Communications (Pty) Ltd. All rights reserved.
#
# The contents of this file are licensed under the MIT License
# (the "License"); you may not use this file except in compliance with the
# License.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
"""prefix_list_agent Package."""

from __future__ import print_function
from __future__ import unicode_literals

import logging

import prefix_list_agent.__meta__  # noqa

from prefix_list_agent.agent import PrefixListAgent
from prefix_list_agent.cli import start

logging.getLogger(__name__).addHandler(logging.NullHandler())

__all__ = [PrefixListAgent.__name__, start.__name__]
